package org.planet.snap.deployer;

import org.planet.snap.Context;
import org.planet.snap.Application;
import org.planet.snap.ApplicationDeploymentException;
import org.planet.snap.ApplicationMetadataIncompleteException;
import org.planet.p2pcm.*;
import org.hsqldb.replication.*;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;

import java.util.Date;
import java.math.BigInteger;

import java.security.KeyPair;
import java.security.KeyStore;
import java.security.Security;
import java.security.PublicKey;
import java.security.Signature;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.KeyPairGenerator;
import java.security.KeyStoreException;
import java.security.SignatureException;
import java.security.InvalidKeyException;
import java.security.NoSuchProviderException;

import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;

import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.x509.X509V1CertificateGenerator;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.StringTokenizer;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import dermi.Registry;
import dermi.exception.*;
import dermi.util.ByteWrapper;

import org.planet.p2pcm.ComponentUtil;

public class SnapBatchDeployer {
  private String adminPassword;
  private final String configFile = "." + File.separator + "WEB-INF" + File.separator + "dermi-config.xml";
  private PublicKey pubKey = null;
  private PrivateKey privKey = null;
  private String p2pURL = null;
  private String fileWAR = null;

  // Constructor
  public SnapBatchDeployer (String[] args) {
    Security.addProvider (new BouncyCastleProvider());
    parseArgs (args);
  }

  public void parseArgs (String[] args) {
    try {
      p2pURL = args[0];
      fileWAR = args[1];

      loadKeyPair();
      insertAndSignWar (fileWAR, p2pURL);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  // Free the allocated resources
  public void close() {
    try {
      if (Registry.isConnected()) {
        Registry.leave();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    System.exit (0);
  }

  public void insertAndSignWar (String warFile, String p2pU) throws ApplicationDeploymentException,
                                                                    ApplicationMetadataIncompleteException,
                                                                    NoSuchAlgorithmException,
                                                                    InvalidKeyException,
                                                                    FileNotFoundException,
                                                                    IOException,
                                                                    SignatureException {

    Signature rsa = Signature.getInstance ("SHA1withRSA");
    rsa.initSign (privKey);

    String tempWarFile = null;
    String p2pUrl = "p2p://snap.deim.urv.net" + p2pU.substring (5, p2pU.length());

    // First update war file (to reflect new assigned p2pUrl)
    try {
      Properties p = extractWarInfo (warFile);
      p.setProperty (Context.SNAP_APPURL, p2pUrl);
      tempWarFile = updateWarFile (warFile, p);
    } catch (Exception e) {
      e.printStackTrace();
      throw new ApplicationDeploymentException ("Unable to update war file.");
    }

    InputStream is = new BufferedInputStream (new FileInputStream (tempWarFile));
    int length = (int) new File (tempWarFile).length();
    byte[] bytes = new byte[length];

    if (length > Integer.MAX_VALUE) {
      // File is too large
      throw new ApplicationDeploymentException ("Application file to sign (" + tempWarFile + ") is too large!");
    }

    // Read in the bytes
    int offset = 0;
    int numRead = 0;
    while (offset < bytes.length
           && (numRead = is.read (bytes, offset, bytes.length - offset)) >= 0) {
      offset += numRead;
    }

    // Ensure all the bytes have been read in
    if (offset < bytes.length) {
      throw new ApplicationDeploymentException ("Could not completely read file " + warFile);
    }

    // Close the input stream and return bytes
    is.close();

    rsa.update (bytes);

    // Compute file signature and deploy web app
    deploySNAPApp (warFile, tempWarFile, p2pUrl, rsa.sign());
  }

  public Properties extractWarInfo (String warFile) throws ApplicationMetadataIncompleteException, ApplicationDeploymentException {
    ZipInputStream bis = null;
    Properties appProperties = new Properties();

    try {
      // Read component's jar file
      bis = new ZipInputStream (new BufferedInputStream (new FileInputStream (warFile)));
      ZipEntry z = bis.getNextEntry();

      // Process all war file entries
      while (z != null) {
        String name = z.getName ();
        name = name.replaceAll ("/", ".");
        if (name.toLowerCase().endsWith ("snap-war.xml")) {
          // Load application metadata
          appProperties.loadFromXML (bis);
          return appProperties;
        }
        z = bis.getNextEntry();
      }
      throw new ApplicationMetadataIncompleteException ("Application metadata is incomplete: WAR lacks 'snap-war.xml' file.");
    } catch (FileNotFoundException fnf) {
      throw new ApplicationDeploymentException ("File not found while trying to extract WAR file.");
    } catch (IOException ioe) {
      throw new ApplicationDeploymentException ("I/O Exception while trying to extract WAR file.");
    } finally {
      try {
        bis.close();
      } catch (Exception e) {}
    }
  }

  private String updateWarFile (String warFile, Properties snapWar_xml) throws ApplicationDeploymentException {
    try {
      ZipOutputStream bos = null;
      ZipInputStream bis = null;

      new File ("./storage").mkdirs();

      bis = new ZipInputStream (new BufferedInputStream (new FileInputStream (warFile)));
      File tempFile = File.createTempFile ("snap", "war", new File("./storage"));
      bos = new ZipOutputStream (new BufferedOutputStream (new FileOutputStream (tempFile)));
      ByteArrayOutputStream byos = new ByteArrayOutputStream();
      snapWar_xml.storeToXML (byos, "SNAP Web-Application descriptor");

      ZipEntry z = bis.getNextEntry();

      byte[] buffer = new byte[1024];
      int bytesRead;

      while (z != null) {
        // Copy all entries from old file to new file except "snap-war.xml"
        if (!z.getName().toLowerCase().endsWith ("snap-war.xml")) {
          bos.putNextEntry (z);
          while ((bytesRead = bis.read (buffer)) != -1) {
            bos.write (buffer, 0, bytesRead);
          }
        }
        z = bis.getNextEntry();
      }
      bis.close();

      // Add modified snap-war.xml
      bos.putNextEntry (new ZipEntry ("/meta-inf/snap-war.xml"));
      ByteArrayInputStream byis = new ByteArrayInputStream (byos.toByteArray());
      while ((bytesRead = byis.read (buffer)) != -1) {
        bos.write (buffer, 0, bytesRead);
      }

      bos.close();
      return tempFile.getAbsolutePath();
    } catch (FileNotFoundException fnf) {
      throw new ApplicationDeploymentException ("File not found while trying to extract WAR file.");
    } catch (IOException ioe) {
      throw new ApplicationDeploymentException ("I/O Exception while trying to extract WAR file.");
    }
  }

  private void deploySNAPApp (String origName, String warFile, String p2pUrl, byte[] appSignature) throws ApplicationDeploymentException,
                                                                                                          ApplicationMetadataIncompleteException {
    Properties appProperties = new Properties();
    boolean snapApp = false;
    ZipInputStream bis = null;
    ArrayList<String> components = new ArrayList<String>();

    try {
      // Read component's jar file
      bis = new ZipInputStream (new BufferedInputStream (new FileInputStream (warFile)));
      ZipEntry z = bis.getNextEntry();

      // Now proceed with the application's registration on the DOLR tree
      if (!Registry.isConnected()) {
        Registry.loadRegistry (new File ("./WEB-INF/dermi-config.xml").getAbsolutePath());
      }

      // Process all war file entries
      while (z != null) {
        String name = z.getName ();
        name = name.replaceAll ("/", ".");
        // System.out.println("load :"+name);
        if (name.toLowerCase().endsWith ("snap-war.xml")) {
          // Load application metadata
          appProperties.loadFromXML (bis);
          appProperties.setProperty (Context.SNAP_APPURL, p2pUrl);

          System.out.println ("\nDeploying SNAP application...");
          System.out.println ("   appName:   " + appProperties.getProperty (Context.SNAP_APPNAME));
          System.out.println ("   appCtx:    " + appProperties.getProperty (Context.SNAP_APPCONTEXT));
          System.out.println ("   appP2PUrl: " + p2pUrl);
          System.out.println ("   warFile:   " + warFile + "\n");

          int i = 1;
          // Keep track of application components
          while (appProperties.getProperty (Context.SNAP_P2PCOMP + i) != null) {
            components.add (appProperties.getProperty (Context.SNAP_P2PCOMP + i));
            i++;
          }

          StringTokenizer tokens = new StringTokenizer (origName, File.separator);
          String war = null;
          while (tokens.hasMoreTokens()) {
            war = tokens.nextToken();
          }
          appProperties.setProperty (Context.SNAP_WARFILE, war);
          snapApp = true;
          break;
        }
        z = bis.getNextEntry();
      }
    } catch (Exception e) {
      ApplicationDeploymentException ade = new ApplicationDeploymentException ("Unable to deploy application due to an unexpected exception: " + warFile);
      e.printStackTrace();
      throw ade;
    } finally {
      try {
        bis.close();
      } catch (Exception e) {}
    }

    if (snapApp) {
      // Check metadata fields are correct for any SNAP application
      if (appProperties.getProperty (Context.SNAP_APPCONTEXT) == null ||
          appProperties.getProperty (Context.SNAP_APPNAME) == null || p2pUrl == null || appSignature == null) {
        if (appProperties.getProperty (Context.SNAP_APPCONTEXT) == null) {
          throw new ApplicationMetadataIncompleteException ("Application's metadata field missing: \"appContext\"");
        }
        if (appProperties.getProperty (Context.SNAP_APPNAME) == null) {
          throw new ApplicationMetadataIncompleteException ("Application's metadata field missing: \"appName\"");
        }
        if (p2pUrl == null) {
          throw new ApplicationMetadataIncompleteException ("Parameter 'p2pUrl' is null!");
        }
        if (appSignature == null) {
          throw new ApplicationMetadataIncompleteException ("Unable to deploy web application: authentication signature is null!");
        }
      }

      try {
        bis = new ZipInputStream (new BufferedInputStream (new FileInputStream (warFile)));
        ZipEntry z = bis.getNextEntry();

        // Process all war file entries
        while (z != null) {
          String name = z.getName ();
          name = name.replaceAll ("/", ".");
          for (Iterator it = components.iterator(); it.hasNext();) {
            if (name.toLowerCase().endsWith ((String) it.next())) {
              // Deploy application component
              System.out.print ("Deploying component " + name + "...");
              ComponentUtil.deployComponent (bis);
              System.out.println ("OK.");
            }
          }
          z = bis.getNextEntry();
        }
      } catch (Exception e) {
        ApplicationDeploymentException ade = new ApplicationDeploymentException ("Unable to deploy application due to an unexpected exception: " + warFile);
        e.printStackTrace();
        throw ade;
      } finally {
        try {
          bis.close ();
        } catch (Exception e) {}
      }

      try {
        // Now proceed with the application's registration on the DOLR tree
        if (!Registry.isConnected()) {
          Registry.loadRegistry (new File ("./WEB-INF/dermi-config.xml").getAbsolutePath());
        }

        // First bind SNAP root if not previously bound
        Application.bindSnapRoot();

        // First check whether this webapp was already bound, and if it is so,
        // stop and remove database trash (if existent)
// COMMENTED FOR NOW!
/*        try {
          this.shutdownDatabases (p2pUrl);
          Registry.unbind ("org.hsqldb.replication.SQLReplicatorFactory$" + p2pUrl + "$Persistence");
        } catch (Exception e) { e.printStackTrace(); }
*/
// PUT SECURE (USING ADMIN PASSWD!!) -> PUBLIC KEY
        Registry.bindSer (Context.SNAP_ADMIN_PK, pubKey);

/*System.out.println ("SIGNATURE IS THIS = ");
for (int idx = 0; idx < appSignature.length; idx++) {
  System.out.print (appSignature[idx]);
}
System.out.println(); */
        
	    Hashtable appMap = new Hashtable();
		Iterator it = appProperties.keySet().iterator();
		while (it.hasNext())  {
			  Object key = it.next();
			  Object value = appProperties.get(key);
			  appMap.put(key,value);
			  
		}
        appMap.put (Context.SNAP_APPSIGNATURE, new ByteWrapper (appSignature));
        Registry.bind (p2pUrl + "_METADATA", appMap);

        // Register WAR file
        InputStream is = new BufferedInputStream (new FileInputStream (warFile));
        int length = (int) new File (warFile).length();
        byte[] bytes = new byte[length];

        if (length > Integer.MAX_VALUE) {
          // File is too large
          throw new ApplicationDeploymentException ("Application file to deploy (" + warFile + ") is too large!");
        }

        // Read in the bytes
        int offset = 0;
        int numRead = 0;
        while (offset < bytes.length
               && (numRead = is.read (bytes, offset, bytes.length - offset)) >= 0) {
          offset += numRead;
        }

        // Ensure all the bytes have been read in
        if (offset < bytes.length) {
          throw new ApplicationDeploymentException ("Could not completely read file " + warFile);
        }

        // Close the input stream and return bytes
        is.close();

        // Delete temporary file
        File f = new File (warFile);
        f.delete();

System.out.println ("LLARGADA WARFILE = " + bytes.length);
//System.out.println ("EL WARFILE = ");
//for (int idx = 0; idx < bytes.length; idx++) {
//   System.out.print (bytes[idx]);
//}
//System.out.println();

        ByteWrapper bw = new ByteWrapper (bytes);
        Registry.bindSer (p2pUrl + "_WARFILE", (Serializable) bw);

/* try {
  System.out.println ("LA VERSIO INSERTADA SIGN = " + Registry.lookupSer ("registry", dermi.util.pastry.HashGenerator.generateHash (p2pUrl + "_METADATA"), "CVS"));
} catch (Exception e) {
  System.out.println ("LA VERSIO INSERTADA SIGN = NOT FOUND");
}
try {
  System.out.println ("LA VERSIO INSERTADA FILE = " + Registry.lookupSer ("registry", dermi.util.pastry.HashGenerator.generateHash (p2pUrl + "_WARFILE"), "CVS"));
} catch (Exception e) {
  System.out.println ("LA VERSIO INSERTADA FILE = NOT FOUND");
} */
        bis.close();

      } catch (Exception ex) {
        ApplicationDeploymentException ade = new ApplicationDeploymentException ("Unable to deploy application: failed to insert metadata into the decentralized registry.");
        ex.printStackTrace();
        throw ade;
      }
    }
  }

  public void shutdownDatabases (String group) {
    ComponentFactory sqlFactory = null;
    ComponentFactory memberFactory = null;

    try {
      sqlFactory = new SQLReplicatorFactory (ComponentUtil.getComponentProperties (SQLReplicatorFactory.class, SQLReplicator.class, SQLReplicatorImpl.class), configFile);
      memberFactory = new MembershipFactory (ComponentUtil.getComponentProperties (MembershipFactory.class, Membership.class, MembershipImpl.class), configFile);

      Membership membership = (Membership) memberFactory.createInstance ("org.hsqldb.replication.Membership", group);
      SQLReplicator sqlRepLocal = (SQLReplicator) sqlFactory.createInstance ("org.hsqldb.replication.SQLReplicator", group);
      SQLReplicator sqlRep = (SQLReplicator) sqlRepLocal.queryInterface ("org.hsqldb.replication.SQLReplicatorRemote");

      sqlRep.shutdown();
      sqlRep.close();
      sqlRepLocal.close();
    } catch (Exception e) {
      // Any exception is to be ignored: if something bad happens, it is likely all databases were automatically
      // shutdown before
      System.out.println ("No databases to shut down.");
    }
  }

  public void loadKeyPair() throws IOException,
                                   FileNotFoundException,
                                   KeyStoreException,
                                   NoSuchAlgorithmException,
                                   CertificateException,
                                   UnrecoverableKeyException {

    FileInputStream fis = new FileInputStream ("keystore.rsa");
    KeyStore ks = KeyStore.getInstance ("PKCS12");
    ks.load (fis, "snap".toCharArray());
    privKey = (PrivateKey) ks.getKey ("admin_privKey", "snap".toCharArray());
    X509Certificate cert = (X509Certificate) ks.getCertificate ("admin_privKey");
    pubKey = cert.getPublicKey();
  }

  /**
   * This method converts a simple HTML-like web application into a WAR file which can be deployed into Snap
   * @param webappDir String Web application directory to transform
   * @param appContext String Web application's desired webserver context
   * @throws FileNotFoundException
   * @throws IOException
   */
  public void transformIntoSnapApp (String webappDir, String appContext) throws FileNotFoundException, IOException {
    File dir = new File (webappDir);
    String[] files = dir.list();
    boolean isJsp = false;

    for (String f : files) {
      if (f.equals ("index.jsp")) {
        isJsp = true;
        File fil = new File (webappDir + File.separator + f);
        File fil2 = new File (webappDir + File.separator + "orig_index.jsp");
        fil.renameTo (fil2);
        break;
      }
      if (f.equals ("index.html") || f.equals ("index.htm")) {
        File fil = new File (webappDir + File.separator + f);
        File fil2 = new File (webappDir + File.separator + "orig_index.html");
        fil.renameTo (fil2);
        break;
      }
    }

    StringBuffer sb = new StringBuffer ("<%@page import=\"dermi.*, org.planet.snap.*, java.util.*, java.io.*, java.net.*;\" %>\n" +
                                        "<%\n\n" +
                                        "// Create and register application\n" +
                                        "Application app = new Application (InetAddress.getLocalHost().getHostName(), request.getServerPort(), Application.getSnapAppConfig (application.getRealPath (File.separator)));\n\n" +
                                        "application.setAttribute (\"snap_webapp\", app);\n\n" +
                                        "%>\n\n" +
                                        "<FRAMESET cols=\"0%, 100%\" FRAMEBORDER=\"NO\" BORDER=\"0\" FRAMESPACING=\"0\">\n" +
                                        "    <FRAME src=\"about:blank\" SCROLLING=\"NO\" noresize>\n" +
                                        "    <FRAME src=\"");

    if (isJsp) {
      sb.append ("orig_index.jsp\"");
    }
    else {
      sb.append ("orig_index.html\"");
    }

    sb.append (" noresize>\n</FRAMESET>\n\n");

    FileOutputStream fos = new FileOutputStream (webappDir + File.separator + "index.jsp");
    fos.write (sb.toString().getBytes());
    fos.close();

    StringBuffer snapWar_xml = new StringBuffer ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                                 "<!DOCTYPE properties SYSTEM \"http://java.sun.com/dtd/properties.dtd\">\n" +
                                                 "<properties>\n" +
                                                 "<comment>\n" +
                                                 "SNAP Web-Application descriptor\n" +
                                                 "</comment>\n" +
                                                 "<entry key=\"appName\">Static HTML web-application template. Converted by SNAPDeployer.</entry>\n" +
                                                 "<entry key=\"appContext\">" + appContext + "</entry>\n" +
                                                 "<entry key=\"appP2PUrl\">p2p://this_entry_is_not_used_now</entry>\n" +
                                                 "</properties>\n");

    File fi = new File (webappDir + File.separator + "meta-inf" + File.separator);
    fi.mkdir();

    fi = new File (webappDir + File.separator + "WEB-INF" + File.separator);
    fi.mkdir();

    fos = new FileOutputStream (webappDir + File.separator + "meta-inf" + File.separator + "snap-war.xml");
    fos.write (snapWar_xml.toString().getBytes());
    fos.close();

    // Let's pack the static webapp into a Snapped war file ;-)
    files = dir.list();

    byte[] buf = new byte[1024];

    try {
      // Create the WAR file
      String outFilename = "." + File.separator + "storage" + File.separator + appContext + ".war";
      ZipOutputStream out = new ZipOutputStream (new FileOutputStream (outFilename));

      // Compress the files
      for (int i = 0; i < files.length; i++) {
        File fff = new File (webappDir + File.separator + files[i]);
        if (fff.isDirectory()) {
          // Add dir files
          addDirFiles (out, fff, files[i]);
        }
        else {
          FileInputStream in = new FileInputStream (fff);

          // Add ZIP entry to output stream.
          out.putNextEntry (new ZipEntry (files[i]));

          // Transfer bytes from the file to the ZIP file
          int len = 0;
          while ((len = in.read (buf)) != -1) {
            out.write (buf, 0, len);
          }

          // Complete the entry
          out.closeEntry();
          in.close();
        }
      }

      // Complete the ZIP file
      out.close();
   } catch (IOException e) {
     e.printStackTrace();
   }

   // Copy 'dermi-config.xml' configuration file
   java.nio.channels.FileChannel in = null, out = null;
   try {
     in = new FileInputStream ("." + File.separator + "WEB-INF" + File.separator + "dermi-config.xml").getChannel();
     out = new FileOutputStream (webappDir + File.separator + "WEB-INF" + File.separator + "dermi-config.xml").getChannel();

     long size = in.size();
     java.nio.MappedByteBuffer bf = in.map (java.nio.channels.FileChannel.MapMode.READ_ONLY, 0, size);
     out.write (bf);
   } finally {
     if (in != null)
       in.close();
     if (out != null)
       out.close();
   }
  }

  /**
   * This method recursively adds a directory to the specified ZipOutputStream
   * @param out ZipOutputStream Output ZIP file
   * @param directory File Directory to add
   * @param dirName String Directory's short name
   * @throws FileNotFoundException
   * @throws IOException
   */
  private void addDirFiles (ZipOutputStream out, File directory, String dirName) throws FileNotFoundException, IOException {
    String[] files = directory.list();
    byte[] buf = new byte[1024];

    for (int i = 0; i < files.length; i++) {
      File fff = new File (directory.getAbsolutePath() + File.separator + files[i]);
      if (fff.isDirectory()) {
        // Add dir files
        addDirFiles (out, fff, files[i]);
      }
      else {
        FileInputStream in = new FileInputStream (fff);

        // Add ZIP entry to output stream.
        out.putNextEntry (new ZipEntry (dirName + File.separator + files[i]));

        // Transfer bytes from the file to the ZIP file
        int len = 0;
        while ((len = in.read (buf)) != -1) {
          out.write (buf, 0, len);
        }

        // Complete the entry
        out.closeEntry();
        in.close();
      }
    }
  }

  public void generateKeyPair() throws NoSuchAlgorithmException,
                                       NoSuchProviderException,
                                       IOException,
                                       KeyStoreException,
                                       CertificateException,
                                       SignatureException,
                                       InvalidKeyException {
    KeyPairGenerator keyGen = KeyPairGenerator.getInstance ("RSA");
    SecureRandom random = SecureRandom.getInstance ("SHA1PRNG", "SUN");
    random.setSeed (System.currentTimeMillis());
    keyGen.initialize (1024, random);

    KeyPair pair = keyGen.generateKeyPair();

    pubKey = pair.getPublic();
    privKey = pair.getPrivate();


    // Generate X.509 certificate
    X509V1CertificateGenerator v1CertGen = new X509V1CertificateGenerator();

    //
    // signers name
    //
    String  issuer = "C=AU, O=Universitat Rovira i Virgili, OU=SNAP Web Application Certificate";

    //
    // subjects name - the same as we are self signed.
    //
    String  subject = "C=AU, O=Universitat Rovira i Virgili, OU=SNAP Web Application Certificate";

    //
    // create the certificate - version 1
    //
    v1CertGen.setSerialNumber(BigInteger.valueOf(10));
    v1CertGen.setIssuerDN(new X509Principal(issuer));
    v1CertGen.setNotBefore(new Date(System.currentTimeMillis() - 1000L * 60 * 60 * 24 * 30));
    v1CertGen.setNotAfter(new Date(System.currentTimeMillis() + (1000L * 60 * 60 * 24 * 30)));
    v1CertGen.setSubjectDN(new X509Principal(subject));
    v1CertGen.setPublicKey(pubKey);
    v1CertGen.setSignatureAlgorithm("SHA1WithRSAEncryption");

    X509Certificate cert = v1CertGen.generateX509Certificate (privKey);

    KeyStore ks = KeyStore.getInstance ("PKCS12");
    ks.load (null, "snap".toCharArray());

    // Aixo igual peta o es insegur pq li falta el certificat a la privKey!!
    ks.setKeyEntry ("admin_privKey", privKey, "snap".toCharArray(), new X509Certificate [] {cert });

    // Store public key and private keys
    FileOutputStream fos = new FileOutputStream ("keystore.rsa");
    ks.store (fos, "snap".toCharArray());
    fos.close();
  }

  public Collection<String> getSnapAppsFromRegistry() throws NotBoundException {
    try {
      if (!Registry.isConnected()) {
        Registry.loadRegistry ("." + File.separator + "WEB-INF" + File.separator + "dermi-config.xml");
      }
      return Application.getApplications();

    } catch (RemoteException re) {
      if (re instanceof NotBoundException) {
        throw (NotBoundException) re;
      }
      re.printStackTrace();
      return new ArrayList<String>();
    }
  }

  // Application's entry point
  public static void main (String[] args) {
    SnapBatchDeployer sd = new SnapBatchDeployer (args);
    sd.close();
  }
}
